import React, {
	useState,
	useReducer,
	MouseEvent,
	FC,
	ChangeEvent,
	FormEvent,
	useEffect,
} from "react";
import * as ReactDOM from "react-dom";
import { offenses } from "./offenses";
import Nav from "./Nav";
//Importing fileReducer to use in useReducer hook, and File interface to use in looping over array of files
import { fileReducer, filesInitialState, File } from "./fileReducer";
import { formReducer, fromInitialState } from "./formReducer";
import "regenerator-runtime/runtime";
import { MapContainer, TileLayer, Marker, useMapEvents } from "react-leaflet";
import "./styles/reset.css";
import "./styles/app.scss";
import { getAddress } from "./location";

const App: FC = () => {
	//This reducer handles adding and selecting files.
	//I'm passing in initial state "{files: []}" from fileReducer file.
	//State can be changed by using dispatch function with object contaning right type and payload.
	//Example: dispatch({type: "ADD_FILE", payload: {img: thisIsImgURL, selected: false})
	//State can be accsesed by using fileState object
	//Example: fileState.files
	//Reducer it self is imported from fileReducer.ts in src/ dir
	const [fileState, fileDispatch] = useReducer(
		fileReducer,
		filesInitialState
	);

	const [formState, formDispatch] = useReducer(formReducer, fromInitialState);

	//This hook handles pining location on the map
	const [mapPin, setMapPin] = useState({ lat: 52.39663, lon: 16.89866 });

	const handleSubmit = async (
		event: FormEvent<HTMLFormElement> | MouseEvent<HTMLButtonElement>
	): Promise<void> => {
		event.preventDefault();
		console.log(fileState, formState);
	};

	const handleUpload = (e: ChangeEvent<HTMLInputElement>) => {
		const files = e.target.files;
		const readAndPreview = (file: any) => {
			// Make sure `file.name` matches our extensions criteria
			if (/\.(jpe?g|png|svg)$/i.test(file.name)) {
				let reader = new FileReader();
				reader.addEventListener(
					"load",
					function () {
						fileDispatch({
							type: "ADD_FILE",
							payload: {
								img: this.result,
								id: Date.now().toString(),
								selected: false,
							},
						});
					},
					false
				);
				reader.readAsDataURL(file);
			}
		};
		if (files) {
			[].forEach.call(files, readAndPreview);
		}
	};
	useEffect(() => {
		getAddress(mapPin).then((blob) =>
			formDispatch({
				type: "CHANGE_FIELD",
				payload: {
					field: "address",
					value: blob,
				},
			})
		);
	}, [mapPin]);
	const MapComponent = () => {
		useMapEvents({
			click: async (e) => {
				setMapPin({ lat: e.latlng.lat, lon: e.latlng.lng });
			},
		});
		return null;
	};
	// console.log(getAddress(mapPin));
	const formMessage = `Rejestracja: ${formState.plate}\nMiejsce: ${
		formState.address.road === undefined ? "" : formState.address.road
	} ${
		formState.address.house_number === undefined
			? ""
			: formState.address.house_number
	}\nData: ${new Date().toLocaleString()}\nMoje dane: ${
		formState.name + ","
	} ${formState.email}\n${
		formState.my_address
	}\nPowód: ${formState.offenses.join(", \n")}\nKomentarz: ${
		formState.comment
	}
  `;

	return (
		<>
			<Nav />
			<div className="container">
				<section className="container__section">
					<h2 className="container__section__title">Zdjęcia</h2>
					<div className="container__section__photos">
						{fileState.files.map((file: File, index: number) => (
							<div
								key={index}
								onClick={(e: MouseEvent) =>
									fileDispatch({
										type: "SELECT_FILE",
										payload: { id: (e.target as any).id },
									})
								}
								className="section__photos__item"
							>
								<img
									src={file.img}
									id={file.id}
									className={
										file.selected ? "img--active" : "img"
									}
								/>
							</div>
						))}
					</div>
					<input
						type="file"
						name="image-upload"
						id="input"
						accept="image/*"
						className="input"
						multiple
						onChange={(e) => handleUpload(e)}
					/>
				</section>
				<section className="container__section">
					<h2 className="container__section__title">Mapa</h2>
					<div>
						<MapContainer
							center={[52.39663, 16.89866]}
							className="container__section__map"
							zoom={16}
							scrollWheelZoom={true}
						>
							<TileLayer
								attribution='&copy; <a href="http://osm.org/copyright">OpenStreetMap</a> contributors'
								url="http://{s}.tile.osm.org/{z}/{x}/{y}.png"
							/>
							<Marker
								position={[mapPin.lat, mapPin.lon]}
							></Marker>
							<MapComponent />
						</MapContainer>
					</div>
					<p>
						Wybrany adres:{" "}
						{formState.address.road === undefined
							? ""
							: formState.address.road}{" "}
						{formState.address.house_number === undefined
							? ""
							: formState.address.house_number}
					</p>
				</section>
				<section className="container__section">
					<h2 className="container__section__title">
						Formularz zgłoszeniowy
					</h2>
					<form
						className="container__section__form"
						onSubmit={(e) => handleSubmit(e)}
					>
						<label htmlFor="name">Twoje imię i nazwisko</label>
						<input
							type="text"
							className="input"
							id="name"
							value={formState.name}
							onChange={(e) =>
								formDispatch({
									type: "CHANGE_FIELD",
									payload: {
										field: e.target.id,
										value: e.target.value,
									},
								})
							}
						/>
						<label htmlFor="email">Twój adres email</label>
						<input
							type="text"
							className="input"
							id="email"
							value={formState.email}
							onChange={(e) =>
								formDispatch({
									type: "CHANGE_FIELD",
									payload: {
										field: e.target.id,
										value: e.target.value,
									},
								})
							}
						/>
						<label htmlFor="my_address">
							Twój adres zamieszkania
						</label>
						<input
							type="text"
							className="input"
							id="my_address"
							value={formState.my_address}
							onChange={(e) =>
								formDispatch({
									type: "CHANGE_FIELD",
									payload: {
										field: e.target.id,
										value: e.target.value,
									},
								})
							}
						/>
						<label htmlFor="plate">
							Numer tablicy rejestracyjnej
						</label>
						<input
							type="text"
							className="input"
							id="plate"
							value={formState.plate}
							onChange={(e) =>
								formDispatch({
									type: "CHANGE_FIELD",
									payload: {
										field: e.target.id,
										value: e.target.value,
									},
								})
							}
						/>
						<div className="container__section__form__offenses">
							{offenses.map((offence, index) => (
								<div key={index}>
									<input
										type="checkbox"
										id={offence.id}
										value={offence.car_is}
										name={offence.car_is}
										className="chceckbox"
										onChange={(e) =>
											e.target.checked === true
												? formDispatch({
														type: "ADD_OFFENCE",
														payload: {
															value:
																e.target.value,
															field: "checkbox",
														},
												  })
												: formDispatch({
														type: "DELETE_OFFENCE",
														payload: {
															value:
																e.target.value,
															field: "checkbox",
														},
												  })
										}
									/>
									<label
										htmlFor={offence.id}
										className="label"
									>
										{offence.name}
									</label>
								</div>
							))}
						</div>
						<label htmlFor="comment">
							Twój komentarz do zgłoszenia
						</label>
						<textarea
							className="textarea textarea--small"
							id="comment"
							name="comment"
							value={formState.comment}
							onChange={(e) =>
								formDispatch({
									type: "CHANGE_FIELD",
									payload: {
										field: e.target.id,
										value: e.target.value,
									},
								})
							}
						/>
						<label htmlFor="message">
							Wiadomość dla straży miejskiej
						</label>
						<textarea
							className="textarea"
							id="message"
							disabled={true}
							value={formMessage}
						/>

						<button
							className="button"
							onClick={(e) => handleSubmit(e)}
						>
							Wysłanie zgłoszenia
						</button>
					</form>
				</section>
			</div>
		</>
	);
};

ReactDOM.render(<App />, document.getElementById("root"));
